/**
 * File: can-shrink.cpp
 * --------------------
 * can-shrink determines whether or not it's 
 * possible to shrink a word down to the empty
 * string.  It's like word-ladder, except that you're
 * removing letters one at a time instead of changing
 * them.
 */

#include <string>
#include <iostream>
using namespace std;

#include "console.h"
#include "stack.h"   // for Stack class
#include "lexicon.h" // for Lexicon class
#include "simpio.h"  // for getLine

/**
 * Function: canShrink
 * -------------------
 * Uses recursive backtracking to decide whether or not the provided
 * string is a word and can be shrunk down to the empty string as per
 * the problem specification.
 */
// suppress warning about unused function by commenting out static keyword
/* static */ bool canShrink(const string& str, const Lexicon& english) {
	if (str.empty()) return true;
    if (!english.contains(str)) return false;
	
    for (size_t i = 0; i < str.size(); i++) {
		string subsequence = str.substr(0, i) + str.substr(i + 1);
		if (canShrink(subsequence, english)) {
			return true;
		}
	}
	
	return false;
}

/**
 * Function: canShrink
 * -------------------
 * Provided the same functionality as the first version of canShrink above, with
 * the bonus that the trail of words proving the provided word can be shrunk down
 * to the empty string is left in the supplied stack.  (If the word can't be
 * shrunk down, then the stack is left empty on return).
 */
static bool canShrink(const string& str, const Lexicon& english, Stack<string>& path) {
	if (str.empty()) {
		path.push("");
		return true;
	}
    
	if (!english.contains(str)) return false;
    for (size_t i = 0; i < str.size(); i++) {
		string subsequence = str.substr(0, i) + str.substr(i + 1);
		if (canShrink(subsequence, english, path)) {
			path.push(str);
			return true;
		}
	}
	
	return false;
}

/**
 * Function: printShrinkPath
 * -------------------------
 * Accepts the presumably nonempty stack and prints the elements in the
 * stack from top to bottom (and in doing so, empties it out).
 */
static void printShrinkPath(Stack<string>& path) {
    cout << endl;
    while (!path.isEmpty()) {
        cout << "\t\"" << path.pop() << "\"" << endl;
    }
    cout << endl;
}

/**
 * Function: main
 * --------------
 * Provides the entry point of the entire program.
 */
int main() {
    Lexicon english("dictionary.txt");
	while (true) {
		string word = getLine("Enter a word: ");
		if (word.empty()) break;
		Stack<string> path;
		if (canShrink(word, english, path)) {
            cout << "Yes! Here's the path: " << endl;
            printShrinkPath(path);
		} else {
			cout << "That word can't be shrunk down." << endl;
		}
	}

    return 0;
}
